import unittest
from xml.etree import ElementTree
from elifetools import xmlio
from elifecleaner import equation


class TestInlineFormulaGraphicHrefs(unittest.TestCase):
    "tests for equation.inline_formula_graphic_hrefs()"

    def test_graphic_hrefs(self):
        "get a list of xlink:href values from inline-formula inline-graphic"
        xml_string = (
            b'<sub-article xmlns:xlink="http://www.w3.org/1999/xlink" id="sa1">'
            b"<body>"
            b'<p>First paragraph with an inline equation <inline-formula id="sa1equ1">'
            b'<inline-graphic xlink:href="elife-sa1-equ1.jpg" />'
            b"</inline-formula>.</p>"
            b"<p>Following is a display formula:</p>"
            b'<p><inline-graphic xlink:href="elife-inf2.jpg" /></p>'
            b"</body>"
            b"</sub-article>"
        )
        identifier = "test.zip"
        tag = ElementTree.fromstring(xml_string)
        expected = ["elife-sa1-equ1.jpg"]
        result = equation.inline_formula_graphic_hrefs(tag, identifier)
        self.assertEqual(result, expected)

    def test_graphic_hrefs_no_match(self):
        "empty list of xlink:href values when there is no matching tag"
        xml_string = (
            b'<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<body><p/></body>"
            b"</sub-article>"
        )
        identifier = "test.zip"
        tag = ElementTree.fromstring(xml_string)
        expected = []
        result = equation.inline_formula_graphic_hrefs(tag, identifier)
        self.assertEqual(result, expected)

    def test_multiple(self):
        "test collecting multiple inline-formula inline-graphic data"
        xml_string = (
            '<sub-article xmlns:xlink="http://www.w3.org/1999/xlink" id="sa1">'
            "<body>"
            "<p>An inline equation"
            ' <inline-formula id="sa1equ1">'
            '<inline-graphic xlink:href="elife-sa1-equ1.jpg" />'
            "</inline-formula>"
            " and another inline equation"
            ' <inline-formula id="sa1equ2">'
            '<inline-graphic xlink:href="elife-sa1-equ2.jpg" />'
            "</inline-formula>"
            ".</p>"
            "</body>"
            "</sub-article>"
        )
        identifier = "test.zip"
        tag = ElementTree.fromstring(xml_string)
        expected = ["elife-sa1-equ1.jpg", "elife-sa1-equ2.jpg"]
        result = equation.inline_formula_graphic_hrefs(tag, identifier)
        self.assertEqual(result, expected)


class TestTransformAll(unittest.TestCase):
    def test_transform_all(self):
        "test transforming to both disp-formula and inline-formula on XML"
        xmlio.register_xmlns()
        sub_article_root = ElementTree.fromstring(
            '<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<body>"
            "<p>First paragraph with an inline equation"
            ' <inline-formula id="sa1equ1"><inline-graphic xlink:href="elife-95901-sa1-equ1.jpg"'
            ' mimetype="image" mime-subtype="jpeg"/>'
            "</inline-formula>.</p>"
            "<p>Inline equation"
            ' <inline-graphic xlink:href="elife-inf2.jpg"/>.</p>'
            "<p>Following is a display formula:</p>"
            '<p><inline-graphic xlink:href="elife-inf3.jpg"/></p>'
            '<p><inline-graphic xlink:href="elife-inf4.jpg"/></p>'
            "<p>An untransformed inline equation"
            ' <inline-graphic xlink:href="elife-inf5.jpg"/>.</p>'
            "</body>"
            "</sub-article>"
        )
        identifier = "10.7554/eLife.95901.1"
        expected = (
            '<sub-article xmlns:xlink="http://www.w3.org/1999/xlink" id="sa1">'
            "<body>"
            "<p>First paragraph with an inline equation"
            ' <inline-formula id="sa1equ1">'
            '<inline-graphic xlink:href="elife-95901-sa1-equ1.jpg"'
            ' mimetype="image" mime-subtype="jpeg" />'
            "</inline-formula>.</p>"
            "<p>Inline equation"
            ' <inline-formula id="sa1equ2">'
            '<inline-graphic xlink:href="elife-sa1-equ2.jpg" />'
            "</inline-formula>.</p>"
            "<p>Following is a display formula:</p>"
            '<disp-formula id="sa1equ3">'
            '<graphic mimetype="image" mime-subtype="jpg" xlink:href="elife-sa1-equ3.jpg" />'
            "</disp-formula>"
            '<disp-formula id="sa1equ4">'
            '<graphic mimetype="image" mime-subtype="jpg" xlink:href="elife-sa1-equ4.jpg" />'
            "</disp-formula>"
            "<p>An untransformed inline equation"
            ' <inline-formula id="sa1equ5">'
            '<inline-graphic xlink:href="elife-sa1-equ5.jpg" />'
            "</inline-formula>.</p>"
            "</body>"
            "</sub-article>"
        )
        # invoke each of the functions
        sub_article_root = equation.transform_equations(sub_article_root, identifier)
        sub_article_root = equation.transform_inline_equations(
            sub_article_root, identifier
        )
        # assert
        self.assertEqual(
            ElementTree.tostring(sub_article_root).decode("utf8"), expected
        )


class TestTransformEquations(unittest.TestCase):
    "tests for transform_equations()"

    def test_transform_equations(self):
        "test converting inline-graphic tags to disp-formula"
        xmlio.register_xmlns()
        sub_article_root = ElementTree.fromstring(
            '<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<body>"
            "<p>First paragraph with an inline equation"
            ' <inline-graphic xlink:href="elife-inf1.jpg"/>.</p>'
            "<p>Following is a display formula:</p>"
            '<p><inline-graphic xlink:href="elife-inf2.jpg"/></p>'
            "</body>"
            "</sub-article>"
        )
        identifier = "10.7554/eLife.95901.1"
        expected = (
            '<sub-article xmlns:xlink="http://www.w3.org/1999/xlink" id="sa1">'
            "<body>"
            "<p>First paragraph with an inline equation"
            ' <inline-graphic xlink:href="elife-inf1.jpg" />.</p>'
            "<p>Following is a display formula:</p>"
            '<disp-formula id="sa1equ2">'
            '<graphic mimetype="image" mime-subtype="jpg" xlink:href="elife-sa1-equ2.jpg" />'
            "</disp-formula>"
            "</body>"
            "</sub-article>"
        )
        # invoke
        result = equation.transform_equations(sub_article_root, identifier)
        # assert
        self.assertEqual(ElementTree.tostring(result).decode("utf8"), expected)

    def test_hybrid(self):
        "test when inline-formula and inline-graphic tags are included"
        xmlio.register_xmlns()
        sub_article_root = ElementTree.fromstring(
            '<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<body>"
            "<p>First paragraph with an inline equation"
            ' <inline-graphic xlink:href="elife-inf1.jpg"/>.</p>'
            "<p>Inline equation"
            ' <inline-formula id="sa1equ1"><inline-graphic xlink:href="elife-95901-sa1-equ1.jpg"'
            ' mimetype="image" mime-subtype="jpeg"/>'
            "</inline-formula>.</p>"
            "<p>Following is a display formula:</p>"
            '<p><inline-graphic xlink:href="elife-inf2.jpg"/></p>'
            "</body>"
            "</sub-article>"
        )
        identifier = "10.7554/eLife.95901.1"
        expected = (
            '<sub-article xmlns:xlink="http://www.w3.org/1999/xlink" id="sa1">'
            "<body>"
            "<p>First paragraph with an inline equation"
            ' <inline-graphic xlink:href="elife-inf1.jpg" />.</p>'
            "<p>Inline equation"
            ' <inline-formula id="sa1equ1"><inline-graphic xlink:href="elife-95901-sa1-equ1.jpg"'
            ' mimetype="image" mime-subtype="jpeg" /></inline-formula>.</p>'
            "<p>Following is a display formula:</p>"
            '<disp-formula id="sa1equ3">'
            '<graphic mimetype="image" mime-subtype="jpg" xlink:href="elife-sa1-equ3.jpg" />'
            "</disp-formula>"
            "</body>"
            "</sub-article>"
        )
        # invoke
        result = equation.transform_equations(sub_article_root, identifier)
        # assert
        self.assertEqual(ElementTree.tostring(result).decode("utf8"), expected)


class TestExtraEquationCount(unittest.TestCase):
    "test for extra_equation_count()"

    def test_inline_graphic(self):
        parent_tag = ElementTree.fromstring(
            '<p xmlns:xlink="http://www.w3.org/1999/xlink">An inline equation'
            ' <inline-graphic xlink:href="elife-inf1.jpg"/>'
            ' and another inline equation <inline-graphic xlink:href="elife-inf2.jpg"/>.</p>'
        )
        expected = 2
        # invoke
        result = equation.extra_equation_count(parent_tag)
        # assert
        self.assertEqual(result, expected)

    def test_inline_formula(self):
        "test counting equation and potential equation tags"
        parent_tag = ElementTree.fromstring(
            '<p xmlns:xlink="http://www.w3.org/1999/xlink">An inline equation'
            ' <inline-formula id="sa0equ1"><inline-graphic xlink:href="elife-95901-sa0-equ1.jpg"'
            ' mimetype="image" mime-subtype="jpeg"/>'
            "</inline-formula>"
            ' and another inline equation <inline-formula id="sa0equ2">'
            '<inline-graphic xlink:href="elife-95901-sa0-equ2.jpg" mimetype="image"'
            ' mime-subtype="jpeg"/'
            "></inline-formula>.</p>"
        )
        expected = 2
        # invoke
        result = equation.extra_equation_count(parent_tag)
        # assert
        self.assertEqual(result, expected)

    def test_disp_formula(self):
        "test counting equation and potential equation tags"
        parent_tag = ElementTree.fromstring(
            '<disp-formula id="sa1equ1" xmlns:xlink="http://www.w3.org/1999/xlink">'
            '<graphic mimetype="image" mime-subtype="jpg" xlink:href="elife-sa1-equ1.jpg" />'
            "</disp-formula>"
        )
        expected = 1
        # invoke
        result = equation.extra_equation_count(parent_tag)
        # assert
        self.assertEqual(result, expected)


class TestDispFormulaTagIndexGroups(unittest.TestCase):
    "tests for disp_formula_tag_index_groups()"

    def test_disp_formula(self):
        "test finding XML to be converted to disp-formula"
        body_tag = ElementTree.fromstring(
            '<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<p>First paragraph with an inline equation"
            ' <inline-graphic xlink:href="elife-inf1.jpg"/>.</p>'
            "<p>Following is a display formula:</p>"
            '<p><inline-graphic xlink:href="elife-inf2.jpg"/></p>'
            "</body>"
        )
        identifier = "10.7554/eLife.95901.1"
        expected = [
            {
                "label_index": None,
                "caption_index": None,
                "inline_graphic_index": 2,
                "tag_index": 2,
            }
        ]
        # invoke
        result = equation.disp_formula_tag_index_groups(body_tag, identifier)
        # assert
        self.assertEqual(result, expected)

    def test_multiple(self):
        "test a sample with multiple inline-formula and disp-formula"
        body_tag = ElementTree.fromstring(
            '<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<p>An inline equation"
            ' <inline-graphic xlink:href="elife-inf1.jpg"/>'
            ' and another inline equation <inline-graphic xlink:href="elife-inf2.jpg"/>.</p>'
            "<p>Following is a display formula:</p>"
            '<p><inline-graphic xlink:href="elife-inf2.jpg"/></p>'
            "</body>"
        )
        identifier = "10.7554/eLife.95901.1"
        expected = [
            {
                "label_index": None,
                "caption_index": None,
                "inline_graphic_index": 2,
                "tag_index": 3,
            }
        ]
        # invoke
        result = equation.disp_formula_tag_index_groups(body_tag, identifier)
        # assert
        self.assertEqual(result, expected)

    def test_hybrid(self):
        "test with an inline-formula tag to count"
        body_tag = ElementTree.fromstring(
            '<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<p>An inline equation"
            ' <inline-formula id="sa0equ1"><inline-graphic xlink:href="elife-95901-sa0-equ1.jpg"'
            ' mimetype="image" mime-subtype="jpeg"/>'
            "</inline-formula>"
            ' and another inline equation <inline-formula id="sa0equ2">'
            '<inline-graphic xlink:href="elife-95901-sa0-equ2.jpg" mimetype="image"'
            ' mime-subtype="jpeg"/'
            "></inline-formula>.</p>"
            "<p>Following is a display formula:</p>"
            '<p><inline-graphic xlink:href="elife-inf2.jpg"/></p>'
            "</body>"
        )
        identifier = "10.7554/eLife.95901.1"
        expected = [
            {
                "label_index": None,
                "caption_index": None,
                "inline_graphic_index": 2,
                "tag_index": 3,
            },
        ]
        # invoke
        result = equation.disp_formula_tag_index_groups(body_tag, identifier)
        # assert
        self.assertEqual(result, expected)


class TestTransformDispFormulas(unittest.TestCase):
    "tests for transform_disp_formulas()"

    def test_transform_disp_formulas(self):
        "convert inline-graphic tags to disp-formula"
        xmlio.register_xmlns()
        body_tag = ElementTree.fromstring(
            '<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<p>First display formula:</p>"
            '<p><inline-graphic xlink:href="elife-inf1.jpg"/></p>'
            "<p>Second display formula:</p>"
            '<p><inline-graphic xlink:href="elife-inf2.jpg"/></p>'
            "</body>"
        )
        index_groups = [
            {
                "inline_graphic_index": 1,
                "tag_index": 1,
            },
            {
                "inline_graphic_index": 3,
                "tag_index": 2,
            },
        ]
        sub_article_id = "sa1"
        expected = (
            '<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<p>First display formula:</p>"
            '<disp-formula id="sa1equ1">'
            '<graphic mimetype="image" mime-subtype="jpg" xlink:href="elife-sa1-equ1.jpg" />'
            "</disp-formula>"
            "<p>Second display formula:</p>"
            '<disp-formula id="sa1equ2">'
            '<graphic mimetype="image" mime-subtype="jpg" xlink:href="elife-sa1-equ2.jpg" />'
            "</disp-formula>"
            "</body>"
        )
        # invoke
        equation.transform_disp_formulas(body_tag, index_groups, sub_article_id)
        # assert
        self.assertEqual(ElementTree.tostring(body_tag).decode("utf8"), expected)


class TestFormulaId(unittest.TestCase):
    "tests for formula_id()"

    def test_formula_id(self):
        "test generating an id for an XML formula or equation"
        sub_article_id = "sa1"
        index = 1
        expected = "sa1equ1"
        # invoke
        result = equation.formula_id(sub_article_id, index)
        # assert
        self.assertEqual(result, expected)


class TestFormulaFileName(unittest.TestCase):
    "tests for formula_file_name()"

    def test_formula_file_name(self):
        "test generating a file name for a equation or formula graphic"
        inf_file_name = "elife-95901-inf1.jpg"
        sub_article_id = "sa1"
        index = "1"
        expected = "elife-95901-sa1-equ1.jpg"
        # invoke
        result = equation.formula_file_name(inf_file_name, sub_article_id, index)
        # assert
        self.assertEqual(result, expected)


class TestTransformDispFormula(unittest.TestCase):
    "tests for transform_disp_formula()"

    def test_transform_disp_formula(self):
        "test converting a single inline-graphic p tag into disp-formula"
        xmlio.register_xmlns()
        body_tag = ElementTree.fromstring(
            '<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<p>First display formula:</p>"
            '<p><inline-graphic xlink:href="elife-inf1.jpg"/></p>'
            "<p>Second display formula:</p>"
            '<p><inline-graphic xlink:href="elife-inf2.jpg"/></p>'
            "</body>"
        )
        index = "2"
        group = {
            "inline_graphic_index": 3,
            "tag_index": 2,
        }
        sub_article_id = "sa1"
        expected = (
            '<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<p>First display formula:</p>"
            '<p><inline-graphic xlink:href="elife-inf1.jpg" /></p>'
            "<p>Second display formula:</p>"
            '<disp-formula id="sa1equ2">'
            '<graphic mimetype="image" mime-subtype="jpg" xlink:href="elife-sa1-equ2.jpg" />'
            "</disp-formula>"
            "</body>"
        )
        # invoke
        equation.transform_disp_formula(body_tag, index, group, sub_article_id)
        # assert
        self.assertEqual(ElementTree.tostring(body_tag).decode("utf8"), expected)


class TestEquationInlineGraphicHrefs(unittest.TestCase):
    "tests for equation_inline_graphic_hrefs()"

    def test_equation_inline(self):
        "test collecting href values for disp-formula"
        sub_article_root = ElementTree.fromstring(
            '<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<body>"
            "<p>First paragraph with an inline equation"
            ' <inline-graphic xlink:href="elife-inf1.jpg"/>.</p>'
            "<p>Following is a display formula:</p>"
            '<p><inline-graphic xlink:href="elife-inf2.jpg"/></p>'
            "</body>"
            "</sub-article>"
        )
        identifier = "10.7554/eLife.95901.1"
        expected = ["elife-inf2.jpg"]
        # invoke
        result = equation.equation_inline_graphic_hrefs(sub_article_root, identifier)
        # assert
        self.assertEqual(result, expected)


class TestFormulaGraphicHrefs(unittest.TestCase):
    "tests for equation.formula_graphic_hrefs()"

    def test_formula_graphic_hrefs(self):
        "get a list of xlink:href values from disp-formula graphic tags"
        xml_string = (
            b'<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink"><body>'
            b"<p>Following is a display formula:</p>\n"
            b'<disp-formula id="sa1equ1">\n'
            b'<graphic mimetype="image" mime-subtype="jpg"'
            b' xlink:href="elife-sa1-equ1.jpg"/>\n'
            b"</disp-formula>\n"
            b"</body></sub-article>"
        )
        identifier = "test.zip"
        tag = ElementTree.fromstring(xml_string)
        expected = ["elife-sa1-equ1.jpg"]
        result = equation.formula_graphic_hrefs(tag, identifier)
        self.assertEqual(result, expected)

    def test_graphic_hrefs_no_match(self):
        "empty list of xlink:href values when there is no graphic tag"
        xml_string = (
            b'<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<body><p/></body>"
            b"</sub-article>"
        )
        identifier = "test.zip"
        tag = ElementTree.fromstring(xml_string)
        expected = []
        result = equation.formula_graphic_hrefs(tag, identifier)
        self.assertEqual(result, expected)


class TestInlineEquationInlineGraphicHrefs(unittest.TestCase):
    "tests for inline_equation_inline_graphic_hrefs()"

    def test_inline_equation_inline(self):
        "test collecting href values for inline-formula"
        sub_article_root = ElementTree.fromstring(
            '<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<body>"
            "<p>First paragraph with an inline equation"
            ' <inline-graphic xlink:href="elife-inf1.jpg"/>.</p>'
            "<p>Following is a display formula:</p>"
            '<p><inline-graphic xlink:href="elife-inf2.jpg"/></p>'
            "</body>"
            "</sub-article>"
        )
        identifier = "10.7554/eLife.95901.1"
        expected = ["elife-inf1.jpg"]
        # invoke
        result = equation.inline_equation_inline_graphic_hrefs(
            sub_article_root, identifier
        )
        # assert
        self.assertEqual(result, expected)


class TestTransformInlineEquations(unittest.TestCase):
    "tests for transform_inline_equations()"

    def test_transform_inline_equations(self):
        "test converting inline-graphic tags to inline-formula"
        xmlio.register_xmlns()
        sub_article_root = ElementTree.fromstring(
            '<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<body>"
            "<p>First paragraph with an inline equation"
            ' <inline-graphic xlink:href="elife-inf1.jpg"/>.</p>'
            "<p>Following is a display formula:</p>"
            '<p><inline-graphic xlink:href="elife-inf2.jpg"/></p>'
            "</body>"
            "</sub-article>"
        )
        identifier = "10.7554/eLife.95901.1"
        expected = (
            '<sub-article xmlns:xlink="http://www.w3.org/1999/xlink" id="sa1">'
            "<body>"
            '<p>First paragraph with an inline equation <inline-formula id="sa1equ1">'
            '<inline-graphic xlink:href="elife-sa1-equ1.jpg" />'
            "</inline-formula>.</p>"
            "<p>Following is a display formula:</p>"
            '<p><inline-graphic xlink:href="elife-inf2.jpg" /></p>'
            "</body>"
            "</sub-article>"
        )
        # invoke
        result = equation.transform_inline_equations(sub_article_root, identifier)
        # assert
        self.assertEqual(ElementTree.tostring(result).decode("utf8"), expected)

    def test_transform_multiple(self):
        "test converting multiple inline-graphic tags to inline-formula"
        xmlio.register_xmlns()
        sub_article_root = ElementTree.fromstring(
            '<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<body>"
            "<p>An inline equation"
            ' <inline-graphic xlink:href="elife-inf1.jpg"/>'
            ' and another inline equation <inline-graphic xlink:href="elife-inf2.jpg"/>.</p>'
            "</body>"
            "</sub-article>"
        )
        identifier = "10.7554/eLife.95901.1"
        expected = (
            '<sub-article xmlns:xlink="http://www.w3.org/1999/xlink" id="sa1">'
            "<body>"
            "<p>An inline equation"
            ' <inline-formula id="sa1equ1">'
            '<inline-graphic xlink:href="elife-sa1-equ1.jpg" />'
            "</inline-formula>"
            " and another inline equation"
            ' <inline-formula id="sa1equ2">'
            '<inline-graphic xlink:href="elife-sa1-equ2.jpg" />'
            "</inline-formula>"
            ".</p>"
            "</body>"
            "</sub-article>"
        )
        # invoke
        result = equation.transform_inline_equations(sub_article_root, identifier)
        # assert
        self.assertEqual(ElementTree.tostring(result).decode("utf8"), expected)


class TestInlineFormulaTagIndexGroups(unittest.TestCase):
    "tests for inline_formula_tag_index_groups()"

    def test_inline_formula(self):
        "test finding XML to be converted to inline-formula"
        body_tag = ElementTree.fromstring(
            '<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<p>First paragraph with an inline equation"
            ' <inline-graphic xlink:href="elife-inf1.jpg"/>.</p>'
            "<p>Following is a display formula:</p>"
            '<p><inline-graphic xlink:href="elife-inf2.jpg"/></p>'
            "</body>"
        )
        identifier = "10.7554/eLife.95901.1"
        expected = [
            {
                "label_index": None,
                "caption_index": None,
                "inline_graphic_index": 0,
                "tag_index": 1,
            }
        ]
        # invoke
        result = equation.inline_formula_tag_index_groups(body_tag, identifier)
        # assert
        self.assertEqual(result, expected)

    def test_multiple(self):
        "test multiple inline-graphic tags in a p tag to be converted to inline-formula"
        body_tag = ElementTree.fromstring(
            '<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<p>An inline equation"
            ' <inline-graphic xlink:href="elife-inf1.jpg"/>'
            ' and another inline equation <inline-graphic xlink:href="elife-inf2.jpg"/>.</p>'
            "</body>"
        )
        identifier = "10.7554/eLife.95901.1"
        expected = [
            {
                "label_index": None,
                "caption_index": None,
                "inline_graphic_index": 0,
                "tag_index": 1,
            },
            {
                "label_index": None,
                "caption_index": None,
                "inline_graphic_index": 0,
                "tag_index": 2,
            },
        ]
        # invoke
        result = equation.inline_formula_tag_index_groups(body_tag, identifier)
        # assert
        self.assertEqual(result, expected)

    def test_hybrid(self):
        "test with a converted disp-formula tag to count"
        body_tag = ElementTree.fromstring(
            '<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            '<disp-formula id="sa0equ1"><graphic /></disp-formula>'
            "<p>An inline equation"
            ' <inline-graphic xlink:href="elife-inf1.jpg"/>'
            ' and another inline equation <inline-graphic xlink:href="elife-inf2.jpg"/>.</p>'
            "</body>"
        )
        identifier = "10.7554/eLife.95901.1"
        expected = [
            {
                "label_index": None,
                "caption_index": None,
                "inline_graphic_index": 1,
                "tag_index": 2,
            },
            {
                "label_index": None,
                "caption_index": None,
                "inline_graphic_index": 1,
                "tag_index": 3,
            },
        ]
        # invoke
        result = equation.inline_formula_tag_index_groups(body_tag, identifier)
        # assert
        self.assertEqual(result, expected)


class TestTransformInlineFormulas(unittest.TestCase):
    "tests for transform_inline_formulas()"

    def test_transform_inline_formulas(self):
        "convert inline-graphic tags to inline-formula"
        xmlio.register_xmlns()
        body_tag = ElementTree.fromstring(
            '<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<p>First inline formula:"
            ' <inline-graphic xlink:href="elife-inf1.jpg"/></p>'
            "<p>Second inline formula:"
            ' <inline-graphic xlink:href="elife-inf2.jpg"/></p>'
            "</body>"
        )
        index_groups = [
            {
                "inline_graphic_index": 0,
                "tag_index": 1,
            },
            {
                "inline_graphic_index": 1,
                "tag_index": 2,
            },
        ]
        sub_article_id = "sa1"
        expected = (
            '<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            '<p>First inline formula: <inline-formula id="sa1equ1">'
            '<inline-graphic xlink:href="elife-sa1-equ1.jpg" />'
            "</inline-formula></p>"
            '<p>Second inline formula: <inline-formula id="sa1equ2">'
            '<inline-graphic xlink:href="elife-sa1-equ2.jpg" />'
            "</inline-formula></p>"
            "</body>"
        )
        # invoke
        equation.transform_inline_formulas(body_tag, index_groups, sub_article_id)
        # assert
        self.assertEqual(ElementTree.tostring(body_tag).decode("utf8"), expected)

    def test_multiple(self):
        "convert multiple inline-graphic tags in a p tag to inline-formula"
        xmlio.register_xmlns()
        body_tag = ElementTree.fromstring(
            '<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<p>An inline equation"
            ' <inline-graphic xlink:href="elife-inf1.jpg"/>'
            ' and another inline equation <inline-graphic xlink:href="elife-inf2.jpg"/>.</p>'
            "</body>"
        )
        index_groups = [
            {
                "inline_graphic_index": 0,
                "tag_index": 1,
            },
            {
                "inline_graphic_index": 0,
                "tag_index": 2,
            },
        ]
        sub_article_id = "sa1"
        expected = (
            '<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<p>An inline equation"
            ' <inline-formula id="sa1equ1">'
            '<inline-graphic xlink:href="elife-sa1-equ1.jpg" />'
            "</inline-formula>"
            " and another inline equation"
            ' <inline-formula id="sa1equ2">'
            '<inline-graphic xlink:href="elife-sa1-equ2.jpg" />'
            "</inline-formula>"
            ".</p>"
            "</body>"
        )
        # invoke
        equation.transform_inline_formulas(body_tag, index_groups, sub_article_id)
        # assert
        self.assertEqual(ElementTree.tostring(body_tag).decode("utf8"), expected)


class TestTransformInlineFormula(unittest.TestCase):
    "tests for transform_inline_formula()"

    def test_transform_inline_formula(self):
        "test converting a single inline-graphic p tag into inline-formula"
        xmlio.register_xmlns()
        body_tag = ElementTree.fromstring(
            '<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<p>First inline formula:"
            ' <inline-graphic xlink:href="elife-inf1.jpg"/></p>'
            "<p>Second inline formula:"
            ' <inline-graphic xlink:href="elife-inf2.jpg"/> and more text.</p>'
            "</body>"
        )
        index = "1"
        group = {
            "inline_graphic_index": 1,
            "tag_index": 2,
        }
        sub_article_id = "sa1"
        expected = (
            '<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<p>First inline formula:"
            ' <inline-graphic xlink:href="elife-inf1.jpg" /></p>'
            "<p>Second inline formula:"
            ' <inline-formula id="sa1equ1">'
            '<inline-graphic xlink:href="elife-sa1-equ1.jpg" />'
            "</inline-formula> and more text.</p>"
            "</body>"
        )
        # invoke
        equation.transform_inline_formula(body_tag, index, group, sub_article_id)
        # assert
        self.assertEqual(ElementTree.tostring(body_tag).decode("utf8"), expected)



import unittest
from xml.etree import ElementTree
from mock import patch
from elifetools import xmlio
from elifecleaner import pub_history
from tests.helpers import read_fixture

SCIETY_DATA = {
    "https://sciety.org/evaluations/hypothesis:6wCSiENREe-fsV9XrL_PJA/content": (
        b"<p><strong>%s</strong></p>\n" b"<p>The ....</p>\n" % b"Author response:"
    ),
    "https://sciety.org/evaluations/hypothesis:63fsPkNREe-RHzPeZyV9vQ/content": (
        b"<p><strong>%s</strong></p>\n"
        b"<p>The ....</p>\n" % b"Reviewer #3 (Public Review):"
    ),
    "https://sciety.org/evaluations/hypothesis:6-3XyENREe-rjbPoWNzSvw/content": (
        b"<p><strong>%s</strong></p>\n"
        b"<p>The ....</p>\n" % b"Reviewer #2 (Public Review):"
    ),
    "https://sciety.org/evaluations/hypothesis:7GBk0kNREe-LnVes3-n_9Q/content": (
        b"<p><strong>%s</strong></p>\n"
        b"<p>The ....</p>\n" % b"Reviewer #1 (Public Review):"
    ),
    "https://sciety.org/evaluations/hypothesis:7NzOvkNREe-fshsElWV5TQ/content": (
        b"<p><strong>%s</strong></p>\n" b"<p>The ....</p>\n" % b"eLife assessment"
    ),
}


def mock_get_web_content(
    url=None,
):
    "return a content containing the response data based on the URL"
    if url and url in SCIETY_DATA:
        return SCIETY_DATA.get(url)
    # default
    return b"<p><strong>%s</strong></p>\n" b"<p>The ....</p>\n" % b"Title"


class TestPruneHistoryData(unittest.TestCase):
    "tests for prune_history_data()"

    def test_prune_history_data(self):
        "test removing history data of newer DOI versions"
        history_data = [
            {"doi": "preprint", "versionIdentifier": 2},
            {"doi": "10.7554/eLife.99854.1", "versionIdentifier": 1},
            {"doi": "10.7554/eLife.99854.2", "versionIdentifier": 2},
            {"doi": "10.7554/eLife.99854.3", "versionIdentifier": 3},
        ]
        doi = "10.7554/eLife.99854"
        version = 3
        expected = history_data[0:3]
        # invoke
        result = pub_history.prune_history_data(history_data, doi, version)
        # assert
        self.assertEqual(result, expected)


class TestFindPubHistoryTag(unittest.TestCase):
    "tests for find_pub_history_tag()"

    def test_find_pub_history_tag(self):
        "test finding pub-history tag from XML"
        xml_string = (
            "<article>"
            "<front>"
            "<article-meta>"
            "<pub-history />"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        root = ElementTree.fromstring(xml_string)
        identifier = "identifier"
        expected = "<pub-history />"
        # invoke
        result = pub_history.find_pub_history_tag(root, identifier)
        # assert
        self.assertEqual(ElementTree.tostring(result).decode("utf-8"), expected)

    def test_history_tag(self):
        "test if history tag is in XML"
        xml_string = (
            "<article>"
            "<front>"
            "<article-meta>"
            "<history />"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        root = ElementTree.fromstring(xml_string)
        identifier = "identifier"
        expected = "<pub-history />"
        # invoke
        result = pub_history.find_pub_history_tag(root, identifier)
        # assert
        self.assertEqual(ElementTree.tostring(result).decode("utf-8"), expected)

    def test_elocation_id_tag(self):
        "test if elocation-id tag is in XML"
        xml_string = (
            "<article>"
            "<front><article-meta><elocation-id /></article-meta></front></article>"
        )
        root = ElementTree.fromstring(xml_string)
        identifier = "identifier"
        expected = "<pub-history />"
        # invoke
        result = pub_history.find_pub_history_tag(root, identifier)
        # assert
        self.assertEqual(ElementTree.tostring(result).decode("utf-8"), expected)

    def test_no_article_meta(self):
        "test no article-meta tag in XML"
        xml_string = "<article />"
        root = ElementTree.fromstring(xml_string)
        identifier = "identifier"
        expected = None
        # invoke
        result = pub_history.find_pub_history_tag(root, identifier)
        # assert
        self.assertEqual(result, expected)


class TestAddSelfUriTag(unittest.TestCase):
    "tests for add_self_uri_tag()"

    def test_add_self_uri_tag(self):
        "test adding self-uri tag"
        xmlio.register_xmlns()
        parent = ElementTree.fromstring("<root />")
        article_type = "preprint"
        uri = "https://example.org/"
        title = "A self URI"
        expected = (
            '<root xmlns:xlink="http://www.w3.org/1999/xlink">'
            '<self-uri content-type="preprint" xlink:href="https://example.org/">'
            "A self URI"
            "</self-uri>"
            "</root>"
        )
        # invoke
        pub_history.add_self_uri_tag(parent, article_type, uri, title)
        # assert
        self.assertEqual(ElementTree.tostring(parent).decode("utf-8"), expected)


class TestAddHistoryEventTag(unittest.TestCase):
    "tests for add_history_event_tag()"

    def test_add_history_event_tag(self):
        "test adding event tag with examples of all content"
        xmlio.register_xmlns()
        parent = ElementTree.fromstring("<root />")
        event_data = {
            "event_desc": "Preprint posted",
            "type": "preprint",
            "doi": "https://example.org/doi",
            "date": "2024-10-21",
            "self_uri_list": [
                {
                    "content_type": "editor-report",
                    "uri": "https://example.org/",
                    "title": "eLife assessment",
                }
            ],
        }
        expected = (
            '<root xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<event>"
            "<event-desc>Preprint posted</event-desc>"
            '<date date-type="preprint" iso-8601-date="2024-10-21">'
            "<day>21</day>"
            "<month>10</month>"
            "<year>2024</year>"
            "</date>"
            '<self-uri content-type="preprint"'
            ' xlink:href="https://doi.org/https://example.org/doi" />'
            '<self-uri content-type="editor-report"'
            ' xlink:href="https://example.org/">eLife assessment</self-uri>'
            "</event>"
            "</root>"
        )
        # invoke
        pub_history.add_history_event_tag(parent, event_data)
        # assert
        self.assertEqual(ElementTree.tostring(parent).decode("utf-8"), expected)


class TestPreprintEventDesc(unittest.TestCase):
    "tests for preprint_event_desc()"

    def test_accepted_style(self):
        "test accepted style"
        style = "accepted"
        expected = "This manuscript was published as a preprint."
        # invoke
        result = pub_history.preprint_event_desc(style)
        # assert
        self.assertEqual(result, expected)

    def test_meca_style(self):
        "test meca style"
        style = "meca"
        expected = "Preprint posted"
        # invoke
        result = pub_history.preprint_event_desc(style)
        # assert
        self.assertEqual(result, expected)

    def test_none(self):
        "test style None"
        style = None
        expected = None
        # invoke
        result = pub_history.preprint_event_desc(style)
        # assert
        self.assertEqual(result, expected)


class TestReviewedPreprintEventDesc(unittest.TestCase):
    "tests for reviewed_preprint_event_desc()"

    def test_accepted_style_first_1(self):
        "test accepted style version 1 first event"
        style = "accepted"
        first_review_event = True
        version = 1
        expected = "This manuscript was published as a reviewed preprint."
        # invoke
        result = pub_history.reviewed_preprint_event_desc(
            style, first_review_event, version
        )
        # assert
        self.assertEqual(result, expected)

    def test_accepted_style_not_first_2(self):
        "test accepted style version 2 not first event"
        style = "accepted"
        first_review_event = False
        version = 2
        expected = "The reviewed preprint was revised."
        # invoke
        result = pub_history.reviewed_preprint_event_desc(
            style, first_review_event, version
        )
        # assert
        self.assertEqual(result, expected)

    def test_meca_style_first_1(self):
        "test meca style version 1 first event"
        style = "meca"
        first_review_event = True
        version = 1
        expected = "Reviewed preprint v1"
        # invoke
        result = pub_history.reviewed_preprint_event_desc(
            style, first_review_event, version
        )
        # assert
        self.assertEqual(result, expected)

    def test_meca_style_not_first_2(self):
        "test meca style version 1 first event"
        style = "meca"
        first_review_event = False
        version = 2
        expected = "Reviewed preprint v2"
        # invoke
        result = pub_history.reviewed_preprint_event_desc(
            style, first_review_event, version
        )
        # assert
        self.assertEqual(result, expected)

    def test_none(self):
        "test style None"
        style = None
        first_review_event = None
        version = None
        expected = None
        # invoke
        result = pub_history.reviewed_preprint_event_desc(
            style, first_review_event, version
        )
        # assert
        self.assertEqual(result, expected)


class TestHistoryEventSelfUriList(unittest.TestCase):
    "tests for history_event_self_uri_list()"

    @patch("docmaptools.parse.get_web_content")
    def test_self_uri_list(self, fake_get):
        "test get self-uri data from docmap"
        fake_get.side_effect = mock_get_web_content
        docmap_string = read_fixture("99854.json", mode="r")
        version_doi = "10.7554/eLife.99854.1"
        expected = [
            {
                "content_type": "editor-report",
                "uri": "https://doi.org/10.7554/eLife.99854.1.sa4",
                "title": "eLife assessment",
            },
            {
                "content_type": "referee-report",
                "uri": "https://doi.org/10.7554/eLife.99854.1.sa3",
                "title": "Reviewer #1 (Public Review):",
            },
            {
                "content_type": "referee-report",
                "uri": "https://doi.org/10.7554/eLife.99854.1.sa2",
                "title": "Reviewer #2 (Public Review):",
            },
            {
                "content_type": "referee-report",
                "uri": "https://doi.org/10.7554/eLife.99854.1.sa1",
                "title": "Reviewer #3 (Public Review):",
            },
            {
                "content_type": "author-comment",
                "uri": "https://doi.org/10.7554/eLife.99854.1.sa0",
                "title": "Author response:",
            },
        ]
        # invoke
        result = pub_history.history_event_self_uri_list(docmap_string, version_doi)
        # assert
        self.assertEqual(result, expected)


HISTORY_DATA_99854 = [
    {
        "type": "preprint",
        "date": "2024-04-27",
        "doi": "10.1101/2024.04.25.591185",
        "url": "https://www.biorxiv.org/content/10.1101/2024.04.25.591185v1",
        "versionIdentifier": "1",
        "published": "2024-04-27",
        "content": [
            {
                "type": "computer-file",
                "url": "s3://prod-elife-epp-meca/99854-v1-meca.zip",
            }
        ],
    },
    {
        "type": "reviewed-preprint",
        "date": "2024-07-17T14:00:00+00:00",
        "identifier": "99854",
        "doi": "10.7554/eLife.99854.1",
        "versionIdentifier": "1",
        "license": "http://creativecommons.org/licenses/by/4.0/",
        "published": "2024-07-17T14:00:00+00:00",
        "partOf": {
            "type": "manuscript",
            "doi": "10.7554/eLife.99854",
            "identifier": "99854",
            "subjectDisciplines": ["Structural Biology and Molecular Biophysics"],
            "published": "2024-07-17T14:00:00+00:00",
            "volumeIdentifier": "13",
            "electronicArticleIdentifier": "RP99854",
            "complement": [],
        },
    },
    {
        "type": "reviewed-preprint",
        "date": "2024-07-18T14:00:00+00:00",
        "identifier": "99854",
        "doi": "10.7554/eLife.99854.2",
        "versionIdentifier": "2",
        "license": "http://creativecommons.org/licenses/by/4.0/",
        "published": "2024-07-18T14:00:00+00:00",
        "partOf": {
            "type": "manuscript",
            "doi": "10.7554/eLife.99854",
            "identifier": "99854",
            "subjectDisciplines": ["Structural Biology and Molecular Biophysics"],
            "published": "2024-07-18T14:00:00+00:00",
            "volumeIdentifier": "13",
            "electronicArticleIdentifier": "RP99854",
            "complement": [],
        },
    },
]


class TestCollectHistoryEventData(unittest.TestCase):
    "tests for collect_history_event_data()"

    def test_accepted(self):
        "test collecting history data for accepted style"
        history_data = [
            {
                "type": "preprint",
                "date": "2022-11-22",
                "doi": "10.1101/2022.11.08.515698",
                "url": "https://www.biorxiv.org/content/10.1101/2022.11.08.515698v2",
                "versionIdentifier": "2",
                "published": "2022-11-22",
                "content": [
                    {
                        "type": "computer-file",
                        "url": (
                            "s3://transfers-elife/biorxiv_Current_Content/November_2022/"
                            "23_Nov_22_Batch_1444/b0f4d90b-6c92-1014-9a2e-aae015926ab4.meca"
                        ),
                    }
                ],
            },
        ]
        style = "accepted"
        docmap_string = read_fixture("99854.json", mode="r")
        add_self_uri = False
        expected = [
            {
                "type": "preprint",
                "date": "2022-11-22",
                "doi": "10.1101/2022.11.08.515698",
                "versionIdentifier": "2",
                "event_desc": "This manuscript was published as a preprint.",
            }
        ]
        # invoke
        result = pub_history.collect_history_event_data(
            history_data, style, docmap_string, add_self_uri
        )
        # assert
        self.assertEqual(len(result), 1)
        self.assertDictEqual(result[0], expected[0])

    @patch("docmaptools.parse.get_web_content")
    def test_meca(self, fake_get):
        "test collecting history data for meca style including self-uri data"
        fake_get.side_effect = mock_get_web_content
        style = "meca"
        docmap_string = read_fixture("99854.json", mode="r")
        add_self_uri = True
        expected = [
            {
                "type": "preprint",
                "date": "2024-04-27",
                "doi": "10.1101/2024.04.25.591185",
                "versionIdentifier": "1",
                "event_desc": "Preprint posted",
                "self_uri_list": [],
            },
            {
                "type": "reviewed-preprint",
                "date": "2024-07-17T14:00:00+00:00",
                "doi": "10.7554/eLife.99854.1",
                "versionIdentifier": "1",
                "event_desc": "Reviewed preprint v1",
                "self_uri_list": [
                    {
                        "content_type": "editor-report",
                        "uri": "https://doi.org/10.7554/eLife.99854.1.sa4",
                        "title": "eLife assessment",
                    },
                    {
                        "content_type": "referee-report",
                        "uri": "https://doi.org/10.7554/eLife.99854.1.sa3",
                        "title": "Reviewer #1 (Public Review):",
                    },
                    {
                        "content_type": "referee-report",
                        "uri": "https://doi.org/10.7554/eLife.99854.1.sa2",
                        "title": "Reviewer #2 (Public Review):",
                    },
                    {
                        "content_type": "referee-report",
                        "uri": "https://doi.org/10.7554/eLife.99854.1.sa1",
                        "title": "Reviewer #3 (Public Review):",
                    },
                    {
                        "content_type": "author-comment",
                        "uri": "https://doi.org/10.7554/eLife.99854.1.sa0",
                        "title": "Author response:",
                    },
                ],
            },
        ]
        # invoke
        result = pub_history.collect_history_event_data(
            HISTORY_DATA_99854[0:2], style, docmap_string, add_self_uri
        )
        # assert
        self.assertEqual(len(result), 2)
        self.assertDictEqual(result[0], expected[0])
        self.assertDictEqual(result[1], expected[1])

    def test_no_history_data(self):
        "test if no history data supplied"
        history_data = []
        expected = []
        # invoke
        result = pub_history.collect_history_event_data(history_data, None, None, None)
        # assert
        self.assertEqual(result, expected)


class TestAddPubHistoryTag(unittest.TestCase):
    "tests for add_pub_history_tag()"

    def test_add_pub_history_tag(self):
        "test adding pub-history tag"
        root = ElementTree.fromstring(
            "<article><front><article-meta /></front></article>"
        )
        history_event_data = [
            {
                "type": "preprint",
                "date": "2022-11-22",
                "doi": "10.1101/2022.11.08.515698",
                "versionIdentifier": "2",
                "event_desc": "Preprint posted",
                "self_uri_list": [],
            },
            {
                "type": "reviewed-preprint",
                "date": "2024-07-17T14:00:00+00:00",
                "doi": "10.7554/eLife.99854.1",
                "versionIdentifier": "1",
                "event_desc": "Reviewed preprint v1",
                "self_uri_list": [
                    {
                        "content_type": "editor-report",
                        "uri": "https://doi.org/10.7554/eLife.99854.1.sa4",
                        "title": "eLife assessment",
                    },
                    {
                        "content_type": "referee-report",
                        "uri": "https://doi.org/10.7554/eLife.99854.1.sa3",
                        "title": "Reviewer #1 (Public Review):",
                    },
                    {
                        "content_type": "referee-report",
                        "uri": "https://doi.org/10.7554/eLife.99854.1.sa2",
                        "title": "Reviewer #2 (Public Review):",
                    },
                    {
                        "content_type": "referee-report",
                        "uri": "https://doi.org/10.7554/eLife.99854.1.sa1",
                        "title": "Reviewer #3 (Public Review):",
                    },
                    {
                        "content_type": "author-comment",
                        "uri": "https://doi.org/10.7554/eLife.99854.1.sa0",
                        "title": "Author response:",
                    },
                ],
            },
        ]
        identifier = "identifier"
        expected = (
            '<article xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<front>"
            "<article-meta>"
            "<pub-history>"
            "<event>"
            "<event-desc>Preprint posted</event-desc>"
            '<date date-type="preprint" iso-8601-date="2022-11-22">'
            "<day>22</day>"
            "<month>11</month>"
            "<year>2022</year>"
            "</date>"
            '<self-uri content-type="preprint"'
            ' xlink:href="https://doi.org/10.1101/2022.11.08.515698" />'
            "</event>"
            "<event>"
            "<event-desc>Reviewed preprint v1</event-desc>"
            '<date date-type="reviewed-preprint" iso-8601-date="2024-07-17">'
            "<day>17</day>"
            "<month>07</month>"
            "<year>2024</year>"
            "</date>"
            '<self-uri content-type="reviewed-preprint"'
            ' xlink:href="https://doi.org/10.7554/eLife.99854.1" />'
            '<self-uri content-type="editor-report"'
            ' xlink:href="https://doi.org/10.7554/eLife.99854.1.sa4">'
            "eLife assessment"
            "</self-uri>"
            '<self-uri content-type="referee-report"'
            ' xlink:href="https://doi.org/10.7554/eLife.99854.1.sa3">'
            "Reviewer #1 (Public Review):"
            "</self-uri>"
            '<self-uri content-type="referee-report"'
            ' xlink:href="https://doi.org/10.7554/eLife.99854.1.sa2">'
            "Reviewer #2 (Public Review):"
            "</self-uri>"
            '<self-uri content-type="referee-report"'
            ' xlink:href="https://doi.org/10.7554/eLife.99854.1.sa1">'
            "Reviewer #3 (Public Review):"
            "</self-uri>"
            '<self-uri content-type="author-comment"'
            ' xlink:href="https://doi.org/10.7554/eLife.99854.1.sa0">'
            "Author response:"
            "</self-uri>"
            "</event>"
            "</pub-history>"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        # invoke
        result = pub_history.add_pub_history_tag(root, history_event_data, identifier)
        # assert
        self.assertEqual(ElementTree.tostring(result).decode("utf-8"), expected)

    def test_no_pub_history_tag(self):
        "test if no pub-history tag can be found or added"
        root = ElementTree.fromstring("<root />")
        history_event_data = [{"foo": "bar"}]
        identifier = "identifier"
        expected = "<root />"
        # invoke
        result = pub_history.add_pub_history_tag(root, history_event_data, identifier)
        # assert
        self.assertEqual(ElementTree.tostring(result).decode("utf-8"), expected)

    def test_no_history_data(self):
        "test if no history event data is supplied"
        root = ElementTree.fromstring("<root />")
        history_event_data = []
        identifier = "identifier"
        expected = "<root />"
        # invoke
        result = pub_history.add_pub_history_tag(root, history_event_data, identifier)
        # assert
        self.assertEqual(ElementTree.tostring(result).decode("utf-8"), expected)


class TestAddPubHistory(unittest.TestCase):
    "tests for prc.add_pub_history()"

    def setUp(self):
        # register XML namespaces
        xmlio.register_xmlns()

        self.xml_string_template = (
            '<article xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<front>"
            "<article-meta>%s</article-meta>"
            "</front>"
            "</article>"
        )
        self.history_data = [
            {
                "type": "preprint",
                "date": "2022-11-22",
                "doi": "10.1101/2022.11.08.515698",
                "url": "https://www.biorxiv.org/content/10.1101/2022.11.08.515698v2",
                "versionIdentifier": "2",
                "published": "2022-11-22",
                "content": [
                    {
                        "type": "computer-file",
                        "url": (
                            "s3://transfers-elife/biorxiv_Current_Content/November_2022/"
                            "23_Nov_22_Batch_1444/b0f4d90b-6c92-1014-9a2e-aae015926ab4.meca"
                        ),
                    }
                ],
            },
            {
                "type": "reviewed-preprint",
                "date": "2023-01-25T14:00:00+00:00",
                "identifier": "85111",
                "doi": "10.7554/eLife.85111.1",
                "versionIdentifier": "1",
                "license": "http://creativecommons.org/licenses/by/4.0/",
                "published": "2023-01-25T14:00:00+00:00",
            },
            {
                "type": "reviewed-preprint",
                "date": "2023-05-10T14:00:00+00:00",
                "identifier": "85111",
                "doi": "10.7554/eLife.85111.2",
                "versionIdentifier": "2",
                "license": "http://creativecommons.org/licenses/by/4.0/",
                "published": "2023-05-10T14:00:00+00:00",
            },
        ]
        self.identifier = "test.zip"
        # expected history XML string for when using the input values
        self.xml_output = (
            "<pub-history>"
            "<event>"
            "<event-desc>This manuscript was published as a preprint.</event-desc>"
            '<date date-type="preprint" iso-8601-date="2022-11-22">'
            "<day>22</day>"
            "<month>11</month>"
            "<year>2022</year>"
            "</date>"
            '<self-uri content-type="preprint"'
            ' xlink:href="https://doi.org/10.1101/2022.11.08.515698" />'
            "</event>"
            "<event>"
            "<event-desc>This manuscript was published as a reviewed preprint.</event-desc>"
            '<date date-type="reviewed-preprint" iso-8601-date="2023-01-25">'
            "<day>25</day>"
            "<month>01</month>"
            "<year>2023</year>"
            "</date>"
            '<self-uri content-type="reviewed-preprint"'
            ' xlink:href="https://doi.org/10.7554/eLife.85111.1" />'
            "</event>"
            "<event>"
            "<event-desc>The reviewed preprint was revised.</event-desc>"
            '<date date-type="reviewed-preprint" iso-8601-date="2023-05-10">'
            "<day>10</day>"
            "<month>05</month>"
            "<year>2023</year>"
            "</date>"
            '<self-uri content-type="reviewed-preprint"'
            ' xlink:href="https://doi.org/10.7554/eLife.85111.2" />'
            "</event>"
            "</pub-history>"
        )

    def test_add_pub_history(self):
        "test adding to an existing pub-history tag"
        xml_string = self.xml_string_template % "<pub-history />"
        expected = bytes(self.xml_string_template % self.xml_output, encoding="utf-8")
        root = ElementTree.fromstring(xml_string)
        root_output = pub_history.add_pub_history(
            root, self.history_data, self.identifier
        )
        self.assertEqual(ElementTree.tostring(root_output), expected)

    def test_no_data(self):
        "test for if there is no data to be added"
        # use xlink:href in the sample and the xmlns is kept in the output
        xml_string = (
            self.xml_string_template % '<ext-link xlink:href="https://example.org" />'
        )
        history_data = None
        expected = bytes(xml_string, encoding="utf-8")
        root = ElementTree.fromstring(xml_string)
        root_output = pub_history.add_pub_history(root, history_data, self.identifier)
        self.assertEqual(ElementTree.tostring(root_output), expected)

    def test_after_history_tag(self):
        "test adding a pub-history tag after an existing history tag"
        xml_string = self.xml_string_template % "<history />"
        expected = bytes(
            self.xml_string_template % ("<history />" + self.xml_output),
            encoding="utf-8",
        )
        root = ElementTree.fromstring(xml_string)
        root_output = pub_history.add_pub_history(
            root, self.history_data, self.identifier
        )
        self.assertEqual(ElementTree.tostring(root_output), expected)

    def test_elocation_id_tag(self):
        "test pub-history tag should be added after the elocation-id tag"
        xml_string = self.xml_string_template % "<elocation-id />"
        expected = bytes(
            self.xml_string_template % ("<elocation-id />" + self.xml_output),
            encoding="utf-8",
        )
        root = ElementTree.fromstring(xml_string)
        root_output = pub_history.add_pub_history(
            root, self.history_data, self.identifier
        )
        self.assertEqual(ElementTree.tostring(root_output), expected)

    def test_no_article_meta(self):
        "test if there is no article-meta tag"
        xml_string = "<article />"
        expected = b"<article />"
        root = ElementTree.fromstring(xml_string)
        root_output = pub_history.add_pub_history(
            root, self.history_data, self.identifier
        )
        self.assertEqual(ElementTree.tostring(root_output), expected)


class TestAddPubHistoryMeca(unittest.TestCase):
    "tests for add_pub_history_meca()"

    def setUp(self):
        v1_events = (
            "<event>"
            "<event-desc>Preprint posted</event-desc>"
            '<date date-type="preprint" iso-8601-date="2024-04-27">'
            "<day>27</day>"
            "<month>04</month>"
            "<year>2024</year>"
            "</date>"
            '<self-uri content-type="preprint"'
            ' xlink:href="https://doi.org/10.1101/2024.04.25.591185" />'
            "</event>"
            "<event>"
            "<event-desc>Reviewed preprint v1</event-desc>"
            '<date date-type="reviewed-preprint" iso-8601-date="2024-07-17">'
            "<day>17</day>"
            "<month>07</month>"
            "<year>2024</year>"
            "</date>"
            '<self-uri content-type="reviewed-preprint"'
            ' xlink:href="https://doi.org/10.7554/eLife.99854.1" />'
            '<self-uri content-type="editor-report"'
            ' xlink:href="https://doi.org/10.7554/eLife.99854.1.sa4">'
            "eLife assessment"
            "</self-uri>"
            '<self-uri content-type="referee-report"'
            ' xlink:href="https://doi.org/10.7554/eLife.99854.1.sa3">'
            "Reviewer #1 (Public Review):"
            "</self-uri>"
            '<self-uri content-type="referee-report"'
            ' xlink:href="https://doi.org/10.7554/eLife.99854.1.sa2">'
            "Reviewer #2 (Public Review):"
            "</self-uri>"
            '<self-uri content-type="referee-report"'
            ' xlink:href="https://doi.org/10.7554/eLife.99854.1.sa1">'
            "Reviewer #3 (Public Review):"
            "</self-uri>"
            '<self-uri content-type="author-comment"'
            ' xlink:href="https://doi.org/10.7554/eLife.99854.1.sa0">'
            "Author response:"
            "</self-uri>"
            "</event>"
        )
        v2_events = (
            "<event>"
            "<event-desc>Reviewed preprint v2</event-desc>"
            '<date date-type="reviewed-preprint" iso-8601-date="2024-07-18">'
            "<day>18</day>"
            "<month>07</month>"
            "<year>2024</year>"
            "</date>"
            '<self-uri content-type="reviewed-preprint"'
            ' xlink:href="https://doi.org/10.7554/eLife.99854.2" />'
            "</event>"
        )
        self.expected_v1_pub_history = (
            "<pub-history>" "%s" "</pub-history>"
        ) % v1_events
        self.expected_v2_pub_history = ("<pub-history>" "%s" "%s" "</pub-history>") % (
            v1_events,
            v2_events,
        )

    @patch("docmaptools.parse.get_web_content")
    def test_add_pub_history_meca(self, fake_get):
        fake_get.side_effect = mock_get_web_content
        root = ElementTree.fromstring(
            "<article>"
            "<front>"
            "<article-meta>"
            "<history />"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        docmap_string = read_fixture("99854.json", mode="r")
        identifier = "10.7554/eLife.99854.2"
        expected = (
            '<article xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<front>"
            "<article-meta>"
            "<history />"
            "%s"
            "</article-meta>"
            "</front>"
            "</article>"
        ) % self.expected_v1_pub_history
        # invoke
        result = pub_history.add_pub_history_meca(
            root, HISTORY_DATA_99854[0:2], docmap_string, identifier
        )
        # assert
        xml_string = ElementTree.tostring(result).decode("utf-8")
        self.assertEqual(xml_string, expected)

    @patch("docmaptools.parse.get_web_content")
    def test_insert_near_elocation_id(self, fake_get):
        "test for inserting pub-history near the elocation-id tag and no history tag"
        fake_get.side_effect = mock_get_web_content
        root = ElementTree.fromstring(
            "<article>"
            "<front>"
            "<article-meta>"
            "<elocation-id />"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        docmap_string = read_fixture("99854.json", mode="r")
        identifier = "10.7554/eLife.99854.2"
        expected = (
            '<article xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<front>"
            "<article-meta>"
            "<elocation-id />"
            "%s"
            "</article-meta>"
            "</front>"
            "</article>"
        ) % self.expected_v1_pub_history
        # invoke
        result = pub_history.add_pub_history_meca(
            root, HISTORY_DATA_99854[0:2], docmap_string, identifier
        )
        # assert
        xml_string = ElementTree.tostring(result).decode("utf-8")
        self.assertEqual(xml_string, expected)

    @patch("docmaptools.parse.get_web_content")
    def test_multiple_history_events(self, fake_get):
        "test more than one version DOI in history data"
        fake_get.side_effect = mock_get_web_content
        root = ElementTree.fromstring(
            "<article>"
            "<front>"
            "<article-meta>"
            "<elocation-id />"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        docmap_string = read_fixture("99854.json", mode="r")
        identifier = "10.7554/eLife.99854.3"
        expected = (
            '<article xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<front>"
            "<article-meta>"
            "<elocation-id />"
            "%s"
            "</article-meta>"
            "</front>"
            "</article>"
        ) % self.expected_v2_pub_history
        # invoke
        result = pub_history.add_pub_history_meca(
            root, HISTORY_DATA_99854, docmap_string, identifier
        )
        # assert
        xml_string = ElementTree.tostring(result).decode("utf-8")
        self.assertEqual(xml_string, expected)

        # self.assertTrue(False)

    def test_no_article_meta(self):
        "test if XML has no article-meta tag"
        xml_string = b"<article />"
        root = ElementTree.fromstring(xml_string)
        history_data = [{"type": "preprint"}]
        docmap_string = {}
        identifier = "10.7554/eLife.95901.1"
        # invoke
        result = pub_history.add_pub_history_meca(
            root, history_data, docmap_string, identifier
        )
        # assert
        self.assertEqual(ElementTree.tostring(result), xml_string)

    def test_no_history_data(self):
        "test if history_data argument is empty"
        xml_string = b"<article />"
        root = ElementTree.fromstring(xml_string)
        history_data = []
        docmap_string = {}
        identifier = "10.7554/eLife.95901.1"
        # invoke
        result = pub_history.add_pub_history_meca(
            root, history_data, docmap_string, identifier
        )
        # assert
        self.assertEqual(ElementTree.tostring(result), xml_string)



import os
import re
from collections import OrderedDict
from xml.etree import ElementTree
import html
from wand.image import Image
from wand.exceptions import PolicyError, WandRuntimeError
from elifearticle import parse as articleparse
from elifetools.utils import escape_ampersand
from elifecleaner import LOGGER, pdf_utils, utils, zip_lib


# flag for whether to try and repair XML if it encounters a ParseError
REPAIR_XML = True

# acceptable file extensions for an art_file
ART_FILE_EXTENSIONS = ["doc", "docx", "tex"]


def article_from_xml(xml_file_path):
    "parse using elifearticle library into an Article object"
    return articleparse.build_article_from_xml(xml_file_path)


def check_ejp_zip(zip_file, tmp_dir):
    "check contents of ejp zip file"
    asset_file_name_map = zip_lib.unzip_zip(zip_file, tmp_dir)
    xml_asset = article_xml_asset(asset_file_name_map)
    root = parse_article_xml(xml_asset[1])
    files = file_list(root)
    # use the zip file name as the identifier for log messages
    identifer = zip_file.split(os.sep)[-1]
    return check_files(files, asset_file_name_map, identifer)


def check_files(files, asset_file_name_map, identifier):
    figures = figure_list(files, asset_file_name_map)
    figures = set_figure_pdf_pages_count(figures)
    # check for multiple page PDF figures
    check_multi_page_figure_pdf(figures, identifier)
    # check for missing files
    check_missing_files(files, asset_file_name_map, identifier)
    # check for file not listed in the manifest
    extra_files = check_extra_files(files, asset_file_name_map, identifier)
    # check for out of sequence files by name
    check_missing_files_by_name(files, identifier)
    # check the art file type
    check_art_file(files, identifier)
    return True


def check_multi_page_figure_pdf(figures, identifier):
    pdfimages_available = pdf_utils.pdfimages_exists()
    for pdf in [pdf for pdf in figures if pdf.get("pages") and pdf.get("pages") > 1]:
        is_multi_page = False
        if pdfimages_available:
            LOGGER.info(
                "%s using pdfimages to check PDF figure file: %s",
                identifier,
                pdf.get("file_name"),
            )
            try:
                image_pages = pdf_utils.pdf_image_pages(pdf.get("file_path"))
                LOGGER.info(
                    "%s pdfimages found images on pages %s in PDF figure file: %s",
                    identifier,
                    image_pages,
                    pdf.get("file_name"),
                )
                is_multi_page = bool([page for page in image_pages if page > 1])
            except:
                LOGGER.exception(
                    "%s exception using pdfimages to check PDF figure file: %s",
                    identifier,
                    pdf.get("file_name"),
                )
                # consider it multi page in the case pdfimages raises an exception
                is_multi_page = True
        else:
            is_multi_page = True
        if is_multi_page:
            LOGGER.warning(
                "%s multiple page PDF figure file: %s",
                identifier,
                pdf.get("file_name"),
            )


def check_missing_files(files, asset_file_name_map, identifier):
    "check for missing files and log a warning if missing"
    missing_files = find_missing_files(files, asset_file_name_map)
    for missing_file in missing_files:
        LOGGER.warning(
            "%s does not contain a file in the manifest: %s",
            identifier,
            missing_file,
        )


def find_missing_files(files, asset_file_name_map):
    "for each file name from the manifest XML file, check for missing files in the zip contents"
    missing_files = []
    asset_file_name_keys = [
        asset_file_key.split("/")[-1] for asset_file_key in asset_file_name_map
    ]
    for manifest_file in files:
        if manifest_file.get("upload_file_nm") not in asset_file_name_keys:
            missing_files.append(manifest_file.get("upload_file_nm"))
    return missing_files


def check_extra_files(files, asset_file_name_map, identifier):
    "check for extra files and log them as a warning if present"
    extra_files = find_extra_files(files, asset_file_name_map)
    for extra_file in extra_files:
        LOGGER.warning(
            "%s has file not listed in the manifest: %s", identifier, extra_file
        )


def find_extra_files(files, asset_file_name_map):
    "check if any file names are missing from the manifest XML"
    extra_files = []

    asset_file_name_keys = [
        asset_file_key.split("/")[-1] for asset_file_key in asset_file_name_map
    ]
    manifest_file_names = [
        manifest_file.get("upload_file_nm")
        for manifest_file in files
        if manifest_file.get("upload_file_nm")
    ]

    # get the name of the article XML file for later
    xml_asset_file_name = None
    xml_asset = article_xml_asset(asset_file_name_map)
    if xml_asset:
        xml_asset_file_name = xml_asset[0].split("/")[-1]

    for file_name in asset_file_name_keys:
        # skip checking for the XML file which is not listed in the manifest
        if file_name == xml_asset_file_name:
            continue
        if file_name not in manifest_file_names:
            extra_files.append(file_name)
    return extra_files


def check_missing_files_by_name(files, identifier):
    "check for files numbered out of sequence and log a warning when found"
    missing_files_by_name = find_missing_files_by_name(files)
    for missing_file in missing_files_by_name:
        LOGGER.warning(
            "%s has file missing from expected numeric sequence: %s",
            identifier,
            missing_file,
        )


def find_missing_files_by_name(files):
    """
    In the manifest file names look for any missing from the expected numeric sequence
    For example, if there is only Figure 1 and Figure 3, Figure 2 is consider to be missing
    """
    missing_files = []
    match_rules = [
        {
            "file_types": ["figure"],
            "meta_names": ["Title", "Figure number"],
            "match_pattern": r"Figure\s+(\d+)",
        }
    ]
    for match_rule in match_rules:

        # collect file values
        file_detail_values = find_file_detail_values(
            files,
            match_rule.get("file_types"),
            match_rule.get("meta_names"),
        )
        meta_values = [file_detail[1] for file_detail in file_detail_values]

        missing_files += find_missing_value_by_sequence(
            meta_values, match_rule.get("match_pattern")
        )
    return missing_files


def find_file_detail_values(files, file_types, meta_names):
    file_detail_values = []
    file_types = list(file_types)
    for file_data in files:
        if file_data.get("file_type") in file_types and file_data.get("custom_meta"):
            for custom_meta in file_data.get("custom_meta"):
                if (
                    custom_meta.get("meta_name")
                    and custom_meta.get("meta_name") in meta_names
                ):
                    # create a tuple of file_type and number
                    file_details = (
                        file_data.get("file_type"),
                        custom_meta.get("meta_value"),
                    )
                    file_detail_values.append(file_details)
                    # only take the first match
                    break
    return file_detail_values


def find_missing_value_by_sequence(values, match_pattern):
    """
    from list of values, use match pattern to collect a numeric sequence and check for
    numbers missing from the sequence
    For example, match_pattern of r"Figure (\\d+)" to get a list of 1, 2, 3, n
    (note: two backslashes are used for one backslash in the above example
     to avoid DeprecationWarning: invalid escape sequence in this comment)
    """
    missing_files = []

    figure_meta_value_match = re.compile(match_pattern)
    label_match = re.compile(r"\(\\d.?\)")

    number_list = []
    for meta_value in values:
        match = figure_meta_value_match.match(meta_value)
        if match:
            number_list.append(int(match.group(1)))

    number_list.sort()

    prev_number = None
    for number in number_list:
        expected_number = None
        if prev_number:
            expected_number = prev_number + 1
        if expected_number and number > expected_number:
            # remove whitespace match pattern
            label_pattern = match_pattern.replace(r"\s+", " ")
            # replace (\d) from the match pattern to get a missing file name
            label = label_match.sub(str(expected_number), label_pattern)
            missing_files.append(label)
        prev_number = number

    return missing_files


def check_art_file(files, identifier):
    "check for an art file and it is an acceptable type"
    art_files = [
        file_data for file_data in files if file_data.get("file_type") == "art_file"
    ]
    file_extensions = [
        utils.file_extension(file_data.get("upload_file_nm"))
        for file_data in art_files
        if file_data.get("upload_file_nm")
    ]
    # convert values to lowercase when comparing
    good_file_extensions = [
        extension
        for extension in file_extensions
        if extension.lower() in ART_FILE_EXTENSIONS
    ]
    if not art_files or not good_file_extensions:
        LOGGER.warning(
            "%s could not find a word or latex article file in the package",
            identifier,
        )


def article_xml_asset(asset_file_name_map):
    """
    find the article XML file name,
    e.g. 30-01-2019-RA-eLife-45644/30-01-2019-RA-eLife-45644.xml
    """
    if not asset_file_name_map:
        return None
    xml_asset = None
    match_pattern = re.compile(r"^(.*)/\1.xml$")
    for asset in asset_file_name_map.items():
        if re.match(match_pattern, asset[0]):
            xml_asset = asset
            break
    return xml_asset


def parse_article_xml(xml_file):
    with open(xml_file, "r") as open_file:
        xml_string = open_file.read()
        # unescape any HTML entities to avoid undefined entity XML exceptions later
        xml_string = html_entity_unescape(xml_string)
        # fix XML-incompatible character entities
        if utils.match_control_character_entities(xml_string):
            LOGGER.info(
                "Replacing character entities in the XML string: %s"
                % utils.match_control_character_entities(xml_string)
            )
            xml_string = utils.replace_control_character_entities(xml_string)
        # also replace unescaped control characters
        if utils.match_control_characters(xml_string):
            LOGGER.info(
                "Replacing control characters in the XML string (ASCII codes): %s"
                % [ord(char) for char in utils.match_control_characters(xml_string)]
            )
            xml_string = utils.replace_control_characters(xml_string)
        try:
            return ElementTree.fromstring(xml_string)
        except ElementTree.ParseError:
            if REPAIR_XML:
                # fix ampersands
                xml_string = escape_ampersand(xml_string)
                # try to repair the xml namespaces
                xml_string = repair_article_xml(xml_string)
                return ElementTree.fromstring(xml_string)
            else:
                LOGGER.exception("ParseError raised because REPAIR_XML flag is False")
                raise


def replace_entity(match):
    "function to use in re.sub for HTMTL entity replacements"
    entity_name = match.group(1)
    ignore_entities = [
        "amp",
        "lt",
        "gt",
    ]
    if entity_name in html.entities.entitydefs and entity_name not in ignore_entities:
        return html.entities.entitydefs[entity_name]
    else:
        return "&%s;" % entity_name


def html_entity_unescape(xml_string):
    "convert HTML entities to unicode characters, except the XML special characters"
    if "&" not in xml_string:
        return xml_string
    match_pattern = re.compile(r"&([^\t\n\f <&#;]{1,32}?);")
    return match_pattern.sub(replace_entity, xml_string)


def repair_article_xml(xml_string):
    if 'xmlns:xlink="http://www.w3.org/1999/xlink"' not in xml_string:
        article_match_pattern = re.compile(r"<article>|<article(\s{1,}.*?)>")
        replacement_pattern = r'<article\1 xmlns:xlink="http://www.w3.org/1999/xlink">'
        return article_match_pattern.sub(
            replacement_pattern,
            xml_string,
        )
    return xml_string


def xml_journal_id_values(root):
    "parse journal-id tags from the XML ElementTree"
    values = {}
    for journal_id_tag in root.findall("./front/journal-meta/journal-id"):
        if journal_id_tag.attrib.get("journal-id-type"):
            values[journal_id_tag.attrib.get("journal-id-type")] = journal_id_tag.text
    return values


def xml_journal_title(root):
    "parse the journal title from the XML ElementTree"
    title = None
    for journal_title_tag in root.findall(
        "./front/journal-meta/journal-title-group/journal-title"
    ):
        title = journal_title_tag.text
        break
    return title


def xml_publisher_name(root):
    "parse the publisher name from the XML ElementTree"
    name = None
    for publisher_name_tag in root.findall(
        "./front/journal-meta/publisher/publisher-name"
    ):
        name = publisher_name_tag.text
        break
    return name


def file_list(root):
    file_list = []
    attribute_map = {
        "file-type": "file_type",
        "id": "id",
    }
    tag_name_map = {
        "upload_file_nm": "upload_file_nm",
    }
    custom_meta_tag_name_map = {
        "meta-name": "meta_name",
        "meta-value": "meta_value",
    }
    for file_tag in root.findall("./front/article-meta/files/file"):
        file_detail = OrderedDict()
        for from_key, to_key in attribute_map.items():
            file_detail[to_key] = file_tag.attrib.get(from_key)
        for from_key, to_key in tag_name_map.items():
            tag = file_tag.find(from_key)
            if tag is not None:
                file_detail[to_key] = tag.text
        custom_meta_tags = tag = file_tag.findall("custom-meta")
        if custom_meta_tags is not None:
            file_detail["custom_meta"] = []
            custom_meta = OrderedDict()
            for custom_meta_tag in custom_meta_tags:
                custom_meta = OrderedDict()
                for from_key, to_key in custom_meta_tag_name_map.items():
                    tag = custom_meta_tag.find(from_key)
                    if tag is not None:
                        custom_meta[to_key] = tag.text
                file_detail["custom_meta"].append(custom_meta)
        file_list.append(file_detail)
    return file_list


def figure_list(files, asset_file_name_map):
    "identify which files are a figure and collect some data about them"
    figures = []

    figure_files = [
        file_data for file_data in files if file_data.get("file_type") == "figure"
    ]

    for file_data in figure_files:
        figure_detail = OrderedDict()
        figure_detail["upload_file_nm"] = file_data.get("upload_file_nm")
        figure_detail["extension"] = file_extension(file_data.get("upload_file_nm"))
        # collect file name data
        for asset_file_name in asset_file_name_map.items():
            if asset_file_name[1].endswith(file_data.get("upload_file_nm")):
                figure_detail["file_name"] = asset_file_name[0]
                figure_detail["file_path"] = asset_file_name[1]
                break
        figures.append(figure_detail)
    return figures


def set_figure_pdf_pages_count(figure_assets):
    "for the pdf files count the number of pages and set the property"
    for figure_detail in figure_assets:
        if figure_detail["extension"] == "pdf":
            figure_detail["pages"] = pdf_page_count(figure_detail.get("file_path"))
    return figure_assets


def file_extension(file_name):
    return file_name.split(".")[-1].lower() if file_name and "." in file_name else None


def pdf_page_count(file_path):
    "open PDF as an image and count the number of pages"
    if file_path:
        try:
            with Image(filename=file_path) as img:
                return len(img.sequence)
        except WandRuntimeError:
            LOGGER.exception(
                "WandRuntimeError in pdf_page_count(), "
                "imagemagick may not be installed"
            )
            raise
        except PolicyError:
            LOGGER.exception(
                "PolicyError in pdf_page_count(), "
                "imagemagick policy.xml may not allow reading PDF files"
            )
            raise
    return None


def preprint_url(root):
    "parse XML to find a preprint URL"
    url = None
    for fn_group_tag in root.findall(
        './front/article-meta/fn-group[@content-type="article-history"]'
    ):
        ext_link_tag = fn_group_tag.find('./ext-link[@ext-link-type="url"]')
        if ext_link_tag is not None:
            url = ext_link_tag.get("{http://www.w3.org/1999/xlink}href")
    return url
